@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of Radicale Server - Calendar Server
-# Copyright © 2008-2010 Guillaume Ayoub
+# Copyright © 2008-2011 Guillaume Ayoub
 # Copyright © 2008 Nicolas Kandel
 # Copyright © 2008 Pascal Halter
 #
@@ -25,8 +25,6 @@ Give a configparser-like interface to read and write configuration.
 
 """
 
-# TODO: Use abstract filenames for other platforms
-
 import os
 import sys
 # Manage Python2/3 different modules
@@ -41,9 +39,9 @@ except ImportError:
 # Default configuration
 INITIAL_CONFIG = {
     "server": {
-        "host": "",
-        "port": "5232",
+        "hosts": "0.0.0.0:5232",
         "daemon": "False",
+        "pid": "",
         "ssl": "False",
         "certificate": "/etc/apache2/ssl/server.crt",
         "key": "/etc/apache2/ssl/server.key"},
@@ -51,12 +49,22 @@ INITIAL_CONFIG = {
         "request": "utf-8",
         "stock": "utf-8"},
     "acl": {
-        "type": "fake",
-        "personal": "False",
-        "filename": "/etc/radicale/users",
-        "encryption": "crypt"},
+        "type": "None",
+        "public_users": "public",
+        "private_users": "private",
+        "httpasswd_filename": "/etc/radicale/users",
+        "httpasswd_encryption": "crypt",
+        "ldap_url": "ldap://localhost:389/",
+        "ldap_base": "ou=users,dc=example,dc=com",
+        "ldap_attribute": "uid",
+        "ldap_binddn": "",
+        "ldap_password": ""},
     "storage": {
-        "folder": os.path.expanduser("~/.config/radicale/calendars")}}
+        "folder": os.path.expanduser("~/.config/radicale/calendars")},
+    "logging": {
+        "config": "/etc/radicale/logging",
+        "debug": "False",
+        "full_environment": "False"}}
 
 # Create a ConfigParser and configure it
 _CONFIG_PARSER = ConfigParser()
@@ -68,6 +76,8 @@ for section, values in INITIAL_CONFIG.items():
 
 _CONFIG_PARSER.read("/etc/radicale/config")
 _CONFIG_PARSER.read(os.path.expanduser("~/.config/radicale/config"))
+if 'RADICALE_CONFIG' in os.environ:
+    _CONFIG_PARSER.read(os.environ['RADICALE_CONFIG'])
 
 # Wrap config module into ConfigParser instance
 sys.modules[__name__] = _CONFIG_PARSER
