@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of Radicale Server - Calendar Server
-# Copyright © 2008-2010 Guillaume Ayoub
+# Copyright © 2008-2011 Guillaume Ayoub
 # Copyright © 2008 Nicolas Kandel
 # Copyright © 2008 Pascal Halter
 #
@@ -25,15 +25,19 @@ Define the main classes of a calendar as seen from the server.
 
 """
 
-import os
 import codecs
+from contextlib import contextmanager
+import json
+import os
+import posixpath
 import time
+import uuid
 
 from radicale import config
 
 
 FOLDER = os.path.expanduser(config.get("storage", "folder"))
-    
+
 
 # This function overrides the builtin ``open`` function for this module
 # pylint: disable=W0622
@@ -49,10 +53,25 @@ def serialize(headers=(), items=()):
     for part in (headers, items):
         if part:
             lines.append("\n".join(item.text for item in part))
-    lines.append("END:VCALENDAR")
+    lines.append("END:VCALENDAR\n")
     return "\n".join(lines)
 
 
+def unfold(text):
+    """Unfold multi-lines attributes.
+
+    Read rfc5545-3.1 for info.
+
+    """
+    lines = []
+    for line in text.splitlines():
+        if lines and (line.startswith(" ") or line.startswith("\t")):
+            lines[-1] += line[1:]
+        else:
+            lines.append(line)
+    return lines
+
+
 class Item(object):
     """Internal iCal item."""
     def __init__(self, text, name=None):
@@ -64,11 +83,11 @@ class Item(object):
         # An item must have a name, determined in order by:
         #
         # - the ``name`` parameter
-        # - the ``X-RADICALE-NAME`` iCal property (for Events and Todos)
-        # - the ``UID`` iCal property (for Events and Todos)
+        # - the ``X-RADICALE-NAME`` iCal property (for Events, Todos, Journals)
+        # - the ``UID`` iCal property (for Events, Todos, Journals)
         # - the ``TZID`` iCal property (for Timezones)
         if not self._name:
-            for line in self.text.splitlines():
+            for line in unfold(self.text):
                 if line.startswith("X-RADICALE-NAME:"):
                     self._name = line.replace("X-RADICALE-NAME:", "").strip()
                     break
@@ -79,14 +98,19 @@ class Item(object):
                     self._name = line.replace("UID:", "").strip()
                     # Do not break, a ``X-RADICALE-NAME`` can appear next
 
-        if "\nX-RADICALE-NAME:" in text:
-            for line in self.text.splitlines():
-                if line.startswith("X-RADICALE-NAME:"):
-                    self.text = self.text.replace(
-                        line, "X-RADICALE-NAME:%s" % self._name)
+        if self._name:
+            if "\nX-RADICALE-NAME:" in text:
+                for line in unfold(self.text):
+                    if line.startswith("X-RADICALE-NAME:"):
+                        self.text = self.text.replace(
+                            line, "X-RADICALE-NAME:%s" % self._name)
+            else:
+                self.text = self.text.replace(
+                    "\nUID:", "\nX-RADICALE-NAME:%s\nUID:" % self._name)
         else:
+            self._name = str(uuid.uuid4())
             self.text = self.text.replace(
-                "\nUID:", "\nX-RADICALE-NAME:%s\nUID:" % self._name)
+                "\nEND:", "\nUID:%s\nEND:" % self._name)
 
     @property
     def etag(self):
@@ -124,6 +148,11 @@ class Todo(Item):
     # pylint: enable=W0511
 
 
+class Journal(Item):
+    """Internal journal class."""
+    tag = "VJOURNAL"
+
+
 class Timezone(Item):
     """Internal timezone class."""
     tag = "VTIMEZONE"
@@ -131,19 +160,87 @@ class Timezone(Item):
 
 class Calendar(object):
     """Internal calendar class."""
-    def __init__(self, path):
-        """Initialize the calendar with ``cal`` and ``user`` parameters."""
-        # TODO: Use properties from the calendar configuration
+    tag = "VCALENDAR"
+
+    def __init__(self, path, principal=False):
+        """Initialize the calendar.
+
+        ``path`` must be the normalized relative path of the calendar, using
+        the slash as the folder delimiter, with no leading nor trailing slash.
+
+        """
         self.encoding = "utf-8"
-        self.owner = path.split("/")[0]
-        self.path = os.path.join(FOLDER, path.replace("/", os.path.sep))
-        # Create calendar if needed, useful for ``self.last_modified``
-        if not os.path.exists(self.path):
-            self.write()
+        split_path = path.split("/")
+        self.path = os.path.join(FOLDER, path.replace("/", os.sep))
+        if principal and split_path and os.path.isdir(self.path):
+            # Already existing principal calendar
+            self.owner = split_path[0]
+        elif len(split_path) > 1:
+            # URL with at least one folder
+            self.owner = split_path[0]
+        else:
+            self.owner = None
+        self.local_path = path if path != '.' else ''
+        self.is_principal = principal
+
+    @classmethod
+    def from_path(cls, path, depth="infinite", include_container=True):
+        """Return a list of calendars and items under the given ``path``.
+
+        If ``depth`` is "0", only the actual object under ``path`` is
+        returned. Otherwise, also sub-items are appended to the result. If
+        ``include_container`` is ``True`` (the default), the containing object
+        is included in the result.
+
+        The ``path`` is relative to the storage folder.
+
+        """
+        # First do normpath and then strip, to prevent access to FOLDER/../
+        sane_path = posixpath.normpath(path.replace(os.sep, "/")).strip("/")
+        attributes = sane_path.split("/")
+        if not attributes:
+            return None
+        if not (os.path.isfile(os.path.join(FOLDER, *attributes)) or
+                path.endswith("/")):
+            attributes.pop()
+
+        result = []
+
+        path = "/".join(attributes)
+        abs_path = os.path.join(FOLDER, path.replace("/", os.sep))
+        principal = len(attributes) <= 1
+        if os.path.isdir(abs_path):
+            if depth == "0":
+                result.append(cls(path, principal))
+            else:
+                if include_container:
+                    result.append(cls(path, principal))
+                try:
+                    for filename in next(os.walk(abs_path))[2]:
+                        if cls.is_vcalendar(os.path.join(abs_path, filename)):
+                            result.append(cls(os.path.join(path, filename)))
+                except StopIteration:
+                    # Directory does not exist yet
+                    pass
+        else:
+            if depth == "0":
+                result.append(cls(path))
+            else:
+                calendar = cls(path, principal)
+                if include_container:
+                    result.append(calendar)
+                result.extend(calendar.components)
+        return result
+
+    @staticmethod
+    def is_vcalendar(path):
+        """Return ``True`` if there is a VCALENDAR file under ``path``."""
+        with open(path) as stream:
+            return 'BEGIN:VCALENDAR' == stream.read(15)
 
     @staticmethod
     def _parse(text, item_types, name=None):
-        """Find items with type in ``item_types`` in ``text`` text.
+        """Find items with type in ``item_types`` in ``text``.
 
         If ``name`` is given, give this name to new items in ``text``.
 
@@ -156,7 +253,7 @@ class Calendar(object):
 
         items = []
 
-        lines = text.splitlines()
+        lines = unfold(text)
         in_item = False
 
         for line in lines:
@@ -191,7 +288,8 @@ class Calendar(object):
         """
         items = self.items
 
-        for new_item in self._parse(text, (Timezone, Event, Todo), name):
+        for new_item in self._parse(
+            text, (Timezone, Event, Todo, Journal), name):
             if new_item.name not in (item.name for item in items):
                 items.append(new_item)
 
@@ -199,10 +297,11 @@ class Calendar(object):
 
     def remove(self, name):
         """Remove object named ``name`` from calendar."""
-        todos = [todo for todo in self.todos if todo.name != name]
-        events = [event for event in self.events if event.name != name]
+        components = [
+            component for component in self.components
+            if component.name != name]
 
-        items = self.timezones + todos + events
+        items = self.timezones + components
         self.write(items=items)
 
     def replace(self, name, text):
@@ -215,20 +314,31 @@ class Calendar(object):
         headers = headers or self.headers or (
             Header("PRODID:-//Radicale//NONSGML Radicale Server//EN"),
             Header("VERSION:2.0"))
-        items = items or self.items
+        items = items if items is not None else self.items
+
+        self._create_dirs(self.path)
 
-        # Create folder if absent
-        if not os.path.exists(os.path.dirname(self.path)):
-            os.makedirs(os.path.dirname(self.path))
-        
         text = serialize(headers, items)
         return open(self.path, "w").write(text)
 
+    @staticmethod
+    def _create_dirs(path):
+        """Create folder if absent."""
+        if not os.path.exists(os.path.dirname(path)):
+            os.makedirs(os.path.dirname(path))
+
     @property
     def etag(self):
         """Etag from calendar."""
         return '"%s"' % hash(self.text)
 
+    @property
+    def name(self):
+        """Calendar name."""
+        with self.props as props:
+            return props.get('D:displayname',
+                self.path.split(os.path.sep)[-1])
+
     @property
     def text(self):
         """Calendar as plain text."""
@@ -242,7 +352,7 @@ class Calendar(object):
         """Find headers items in calendar."""
         header_lines = []
 
-        lines = self.text.splitlines()
+        lines = unfold(self.text)
         for line in lines:
             if line.startswith("PRODID:"):
                 header_lines.append(Header(line))
@@ -255,7 +365,12 @@ class Calendar(object):
     @property
     def items(self):
         """Get list of all items in calendar."""
-        return self._parse(self.text, (Event, Todo, Timezone))
+        return self._parse(self.text, (Event, Todo, Journal, Timezone))
+
+    @property
+    def components(self):
+        """Get list of all components in calendar."""
+        return self._parse(self.text, (Event, Todo, Journal))
 
     @property
     def events(self):
@@ -267,6 +382,11 @@ class Calendar(object):
         """Get list of ``Todo`` items in calendar."""
         return self._parse(self.text, (Todo,))
 
+    @property
+    def journals(self):
+        """Get list of ``Journal`` items in calendar."""
+        return self._parse(self.text, (Journal,))
+
     @property
     def timezones(self):
         """Get list of ``Timezome`` items in calendar."""
@@ -279,5 +399,38 @@ class Calendar(object):
         The date is formatted according to rfc1123-5.2.14.
 
         """
+        # Create calendar if needed
+        if not os.path.exists(self.path):
+            self.write()
+
         modification_time = time.gmtime(os.path.getmtime(self.path))
         return time.strftime("%a, %d %b %Y %H:%M:%S +0000", modification_time)
+
+    @property
+    @contextmanager
+    def props(self):
+        """Get the calendar properties."""
+        props_path = self.path + '.props'
+        # On enter
+        properties = {}
+        if os.path.exists(props_path):
+            with open(props_path) as prop_file:
+                properties.update(json.load(prop_file))
+        yield properties
+        # On exit
+        self._create_dirs(props_path)
+        with open(props_path, 'w') as prop_file:
+            json.dump(properties, prop_file)
+
+    @property
+    def owner_url(self):
+        """Get the calendar URL according to its owner."""
+        if self.owner:
+            return "/%s/" % self.owner
+        else:
+            return None
+
+    @property
+    def url(self):
+        """Get the standard calendar URL."""
+        return "/%s/" % self.local_path
